@@ -6,7 +6,7 @@ use strict;
 use Carp;
 use Statistics::Lite qw(mean);
 use Bio::DB::BigBed;
-our $VERSION = '1.14';
+our $VERSION = '1.15';
 
 
 # Exported names
@@ -18,12 +18,6 @@ our @EXPORT = qw(
 	sum_total_bigbed_features
 );
 
-# Hashes of opened file objects
-our %OPENED_BEDFILES; # opened bigbed file objects
-	# in empirical testing, this doesn't really seem to speed things up
-	# like I thought it would
-	# oh well, keep it anyway????
-
 # Hash of Bigfile chromosomes
 our %BIGBED_CHROMOS;
 	# sometimes user may request a chromosome that's not in the bigfile
@@ -31,6 +25,10 @@ our %BIGBED_CHROMOS;
 	# we will record the chromosomes list in this hash
 	# $BIGBED_CHROMOS{bigfile}{chromos}
 
+# Opened bigBed db objects
+our %OPENED_BB;
+	# a cache for opened BigBed databases, primarily for collecting scores
+
 # The true statement
 1; 
 
@@ -56,12 +54,22 @@ sub collect_bigbed_scores {
 	
 	# look at each bedfile
 	# usually there is only one, but for stranded data there may be 
-	# two bedfiles (+ and -), so we'll check each wig file for strand info
+	# two bedfiles (+ and -), so we'll check each bed file for strand info
 	foreach my $bedfile (@bed_features) {
 	
 		# open the bedfile
-		my $bb = open_bigbed_db($bedfile) or 
-			croak "Unable to open bigBed file '$bedfile'! $!\n";
+		my $bb;
+		if (exists $OPENED_BB{$bedfile}) {
+			# use a cached object
+			$bb = $OPENED_BB{$bedfile};
+		}
+		else {
+			# open and cache the bigWig object
+			$bb = open_bigbed_db($bedfile) or 
+				croak " Unable to open bigBed file '$bedfile'! $!\n";
+			$OPENED_BB{$bedfile} = $bb;
+			%{ $BIGBED_CHROMOS{$bedfile} } = map { $_ => 1 } $bb->seq_ids;
+		}
 			
 		# first check that the chromosome is present
 		unless (exists $BIGBED_CHROMOS{$bedfile}{$chromo}) {
@@ -132,12 +140,22 @@ sub collect_bigbed_position_scores {
 	
 	# look at each bedfile
 	# usually there is only one, but for stranded data there may be 
-	# two bedfiles (+ and -), so we'll check each wig file for strand info
+	# two bedfiles (+ and -), so we'll check each bed file for strand info
 	foreach my $bedfile (@bed_features) {
 	
-		# check for opened bedfile
-		my $bb = open_bigbed_db($bedfile) or 
-			croak "Unable to open bigBed file '$bedfile'! $!\n";
+		# open the bedfile
+		my $bb;
+		if (exists $OPENED_BB{$bedfile}) {
+			# use a cached object
+			$bb = $OPENED_BB{$bedfile};
+		}
+		else {
+			# open and cache the bigWig object
+			$bb = open_bigwig_db($bedfile) or 
+				croak " Unable to open bigBed file '$bedfile'! $!\n";
+			$OPENED_BB{$bedfile} = $bb;
+			%{ $BIGBED_CHROMOS{$bedfile} } = map { $_ => 1 } $bb->seq_ids;
+		}
 			
 		# first check that the chromosome is present
 		unless (exists $BIGBED_CHROMOS{$bedfile}{$chromo}) {
@@ -228,35 +246,19 @@ sub collect_bigbed_position_scores {
 ### Open a bigBed database connection
 sub open_bigbed_db {
 	
+	# check path
 	my $bedfile = shift;
-	my $forget  = shift;
+	my $path = $bedfile;
+	$path =~ s/^file://; # clean up file prefix if present
 	
-	# check whether the file has been opened or not
-	if (exists $OPENED_BEDFILES{$bedfile} ) {
-		# this file is already opened, use it
-		return $OPENED_BEDFILES{$bedfile};
-	}
+	# open
+	my $bb;
+	eval {
+		$bb = Bio::DB::BigBed->new($path);
+	};
+	return unless $bb;
 	
-	else {
-		# this file has not been opened yet, open it
-		my $path = $bedfile;
-		$path =~ s/^file://; # clean up file prefix if present
-		my $bb;
-		eval {
-			$bb = Bio::DB::BigBed->new($path);
-		};
-		return unless $bb;
-		
-		unless ($forget) {
-			# store the opened object for later use
-			$OPENED_BEDFILES{$bedfile} = $bb;
-		
-			# collect the chromosomes for this bigBed file
-			%{ $BIGBED_CHROMOS{$bedfile} } = map { $_ => 1 } $bb->seq_ids;
-		}
-		
-		return $bb;
-	}
+	return $bb;
 }
 
 
@@ -282,7 +284,7 @@ sub sum_total_bigbed_features {
 	else {
 		# we have a name of a bigbed file
 		# open it but do not remember it
-		$bb = open_bigbed_db($bb_file, 1);
+		$bb = open_bigbed_db($bb_file);
 		return unless ($bb);
 	}
 	
@@ -317,10 +319,6 @@ feature is collected.
 For loading bigbed files into a Bio::DB database, see the biotoolbox perl 
 script 'big_filegff3.pl'.
 
-To speed up the program and avoid repetitive opening and 
-closing of the files, the opened bigbed file object is stored in a global 
-hash in case it is needed again.
-
 =head1 USAGE
 
 The module requires Lincoln Stein's Bio::DB::BigBed to be installed. 
