@@ -5,11 +5,13 @@
 use strict;
 use Getopt::Long;
 use Pod::Usage;
+use FindBin qw($Bin);
 use Statistics::Lite qw(sum min max mean median stddevp);
 use Statistics::Descriptive;
 use Bio::ToolBox::data_helper qw(
 	find_column_index
 	format_with_commas
+	splice_data_structure
 );
 use Bio::ToolBox::db_helper qw(
 	open_db_connection
@@ -21,7 +23,26 @@ use Bio::ToolBox::file_helper qw(
 	load_tim_data_file 
 	write_tim_data_file 
 );
-my $VERSION = '1.15';
+my $parallel;
+eval {
+	# check for parallel support
+	require Parallel::ForkManager;
+	$parallel = 1;
+};
+
+# ANOVA Parameters for P-value Statistics
+my $anova;
+eval {
+	# check for ANOVA support
+	require Statistics::ANOVA;
+	$anova = 1;
+};
+my $INDEPENDENCE  = 0;
+my $PARAMETRIC    = 1;
+my $ORDINAL       = 0;
+	
+use constant LOG10 => log(10);
+my $VERSION = '1.17';
 
 print "\n This program will correlate positions of occupancy between two datasets\n\n";
 
@@ -45,6 +66,7 @@ my (
 	$data_database,
 	$refDataSet,
 	$testDataSet,
+	$find_pvalue,
 	$find_shift,
 	$radius,
 	$position,
@@ -52,6 +74,7 @@ my (
 	$norm_method,
 	$interpolate,
 	$gz,
+	$cpu,
 	$help,
 	$print_version,
 );
@@ -64,6 +87,7 @@ GetOptions(
 	'ddb=s'       => \$data_database, # database containing datasets
 	'ref=s'       => \$refDataSet, # reference dataset
 	'test=s'      => \$testDataSet, # test dataset
+	'pval!'       => \$find_pvalue, # calculate student t-test
 	'shift!'      => \$find_shift, # calculate optimum shift
 	'radius=i'    => \$radius, # for collecting data when shifting
 	'pos=s'       => \$position, # set the relative feature position
@@ -72,6 +96,7 @@ GetOptions(
 	'norm=s'      => \$norm_method, # method of normalization
 	'interpolate!' => \$interpolate, # interpolate the position data
 	'gz!'         => \$gz, # compress output
+	'cpu=i'      => \$cpu, # number of execution threads
 	'help'        => \$help, # request help
 	'version'     => \$print_version, # print the version
 ) or die " unrecognized option(s)!! please refer to the help documentation\n\n";
@@ -94,47 +119,7 @@ if ($print_version) {
 
 
 ### Check for requirements
-unless ($infile) {
-	$infile = shift @ARGV or
-		die " no input file! use --help for more information\n";
-}
-unless (defined $gz) {
-	$gz = 0;
-}
-
-if (defined $position) {
-	# check the position value
-	unless (
-		$position == 5 or
-		$position == 3 or
-		$position == 4 or
-		$position eq 'm'
-	) {
-		die " Unknown relative position '$position'!\n";
-	}
-	if ($position eq 'm') {$position = 4} # change to match internal usage
-}
-else {
-	# default position to use the midpoint
-	$position = 4;
-}
-
-unless (defined $interpolate) {
-	$interpolate = 1;
-}
-
-if ($norm_method) {
-	unless ($norm_method =~ /^rank|sum$/i) {
-		die " Unrecognized normalization method! see help\n";
-	}
-}
-
-if ($find_shift) {
-	unless ($radius) {
-		die " Must define a radius and reference point to calculate optimum shift\n";
-	}
-}
-
+check_defaults();
 
 
 
@@ -145,22 +130,19 @@ my $mainData = load_tim_data_file($infile) or
 
 
 ### Open database connection
+my ($db, $ddb);
 unless ($database) {
-	$database = $mainData->{'db'} or 
-		die " no database provided! use --help for more information\n";
+	$database = $mainData->{'db'} || undef;
+}
+if ($database) {
+	$db = open_db_connection($database) or 
+		die " unable to open database '$database' connection!\n";
 }
-my $db = open_db_connection($database) or 
-	die " unable to open database '$database' connection!\n";
-my $ddb;
 if ($data_database) {
 	# separate database for the refDataSet and testDataSet
 	$ddb = open_db_connection($data_database) or 
 		die " unable to open data database '$data_database' connection!\n";
 }
-else {
-	# use the same database
-	$ddb = $db;
-}
 
 
 
@@ -170,34 +152,33 @@ validate_or_request_dataset();
 
 ### Identify column indices
 my %index = identify_indices();
-# Prepare new columns
-add_new_columns();
 
 
-### Collect correlations
+
+### Collect the data correlations
 my $start_time = time;
-
-print " Collecting correlations....\n";
-collect_correlations();
-
-
-
-### Output file
-unless ($outfile) {
-	$outfile = $infile;
+# check whether it is worth doing parallel execution
+if ($cpu > 1) {
+	while ($cpu > 1 and $mainData->{'last_row'}/$cpu < 100) {
+		# I figure we need at least 100 lines in each fork split to make 
+		# it worthwhile to do the split, otherwise, reduce the number of 
+		# splits to something more worthwhile
+		$cpu--;
+	}
 }
-my $success = write_tim_data_file(
-	'data'     => $mainData,
-	'filename' => $outfile,
-	'gz'       => $gz,
-);
-if ($success) {
-	print " wrote output file $success\n";
+
+if ($cpu > 1) {
+	# parallel execution
+	parallel_execution();
 }
 else {
-	print " unable to write output file!\n";
+	# single process execution
+	single_execution();
 }
 
+
+
+
 # The End
 printf " Finished in %.2f minutes\n", (time - $start_time) / 60;
 
@@ -206,6 +187,72 @@ printf " Finished in %.2f minutes\n", (time - $start_time) / 60;
 
 ########################   Subroutines   ###################################
 
+sub check_defaults {
+	
+	unless ($infile) {
+		$infile = shift @ARGV or
+			die " no input file! use --help for more information\n";
+	}
+	unless (defined $gz) {
+		$gz = 0;
+	}
+
+	if (defined $position) {
+		# check the position value
+		unless (
+			$position == 5 or
+			$position == 3 or
+			$position == 4 or
+			$position eq 'm'
+		) {
+			die " Unknown relative position '$position'!\n";
+		}
+		if ($position eq 'm') {$position = 4} # change to match internal usage
+	}
+	else {
+		# default position to use the midpoint
+		$position = 4;
+	}
+
+	unless (defined $interpolate) {
+		$interpolate = 1;
+	}
+
+	if ($norm_method) {
+		unless ($norm_method =~ /^rank|sum$/i) {
+			die " Unrecognized normalization method! see help\n";
+		}
+	}
+
+	if ($find_pvalue) {
+		unless ($anova) {
+			die " Cannot calculate requested P-value! Pleast install Statistics::ANOVA\n";
+		}
+	}
+
+	if ($find_shift) {
+		unless ($radius) {
+			die " Must define a radius and reference point to calculate optimum shift\n";
+		}
+	}
+
+	if ($parallel) {
+		# conservatively enable 2 cores
+		$cpu ||= 2;
+	}
+	else {
+			# disable cores
+			print " disabling parallel CPU execution, no support present\n" if $cpu;
+			$cpu = 0;
+	}
+
+	unless ($outfile) {
+		$outfile = $infile;
+	}
+
+}
+
+
 sub validate_or_request_dataset {
 	
 	# Process the reference dataset
@@ -257,6 +304,120 @@ sub identify_indices {
 
 
 
+sub parallel_execution {
+	my $pm = Parallel::ForkManager->new($cpu);
+	
+	# generate base name for child processes
+	my $child_base_name = $outfile . ".$$"; 
+	
+	# variables for reporting the summary of results
+	my @correlations;
+	my @optimal_shifts;
+	my @optimal_correlations;
+	my $count = 0;
+	my $not_enough_data = 0;
+	my $no_variance = 0;
+	$pm->run_on_finish( sub {
+		my ($pid, $exit_code, $ident, $exit_signal, $core_dump, $result) = @_;
+		push @correlations, @{ $result->[0] };
+		push @optimal_shifts, @{ $result->[1] };
+		push @optimal_correlations, @{ $result->[2] };
+		$count += $result->[3];
+		$not_enough_data += $result->[4];
+		$no_variance += $result->[5];
+	} );
+
+	# Split the input data into parts and execute in parallel in separate forks
+	print " Forking into $cpu children for parallel correlation collections\n";
+	for my $i (1 .. $cpu) {
+		$pm->start and next;
+	
+		#### In child ####
+	
+		# splice the data structure
+		splice_data_structure($mainData, $i, $cpu);
+		
+		# re-open database objects to make them clone safe
+		if ($database) {
+			$db = open_db_connection($database);
+		}
+		if ($data_database) {
+			$ddb = open_db_connection($data_database);
+		}
+		
+		# Prepare new columns
+		add_new_columns();
+	
+		# collect
+		my @results = collect_correlations();
+
+		# write output file
+		my $success = write_tim_data_file(
+			'data'     => $mainData,
+			'filename' => "$child_base_name.$i",
+			'gz'       => 0, # faster to write without compression
+		);
+		if ($success) {
+			print " wrote output file $success\n";
+		}
+		else {
+			# failure! the subroutine will have printed error messages
+			die " unable to write file!\n";
+			# no need to continue
+		}
+		
+		# Finished
+		$pm->finish(0, \@results); 
+	}
+	$pm->wait_all_children;
+	
+	# summarize the results
+	summarize_results(\@correlations, \@optimal_shifts, \@optimal_correlations,
+		$count, $not_enough_data, $no_variance);
+	
+	# reassemble children files into output file
+	my @files = glob "$child_base_name.*";
+	unless (@files) {
+		die "unable to find children files!\n";
+	}
+	my @args = ("$Bin/join_data_file.pl", "--out", $outfile);
+	push @args, '--gz' if $gz;
+	push @args, @files;
+	system(@args) == 0 or die " unable to execute join_data_file.pl! $?\n";
+	unlink @files;
+	# done
+}
+
+
+
+sub single_execution {
+	# run in a single thread
+	
+	# Prepare new columns
+	add_new_columns();
+	
+	# collect
+	print " Collecting correlations....\n";
+	my @results = collect_correlations();
+	
+	# summarize
+	summarize_results(@results);
+	
+	# write output file
+	my $success = write_tim_data_file(
+		'data'     => $mainData,
+		'filename' => $outfile,
+		'gz'       => $gz,
+	);
+	if ($success) {
+		print " wrote output file $success\n";
+	}
+	else {
+		print " unable to write output file!\n";
+	}
+}
+
+
 
 sub collect_correlations {
 	
@@ -275,7 +436,6 @@ sub collect_correlations {
 		# using named features
 		# retrieve the coordinates from the database
 		$collect_coordinates = \&collect_coordinates_from_db;
-		print "  coordinates will be collected from database $database\n";
 	}
 	elsif (
 		defined $index{chrom} and 
@@ -284,7 +444,6 @@ sub collect_correlations {
 	) {
 		# coordinates are defined in the input file
 		$collect_coordinates = \&collect_coordinates_from_file;
-		print "  coordinates will be collected from file\n";
 	}
 	else {
 		# don't have enough info to continue
@@ -336,6 +495,8 @@ sub collect_correlations {
 				'chromo'    => $chromo,
 				'start'     => $ref_point - (2 * $radius),
 				'stop'      => $ref_point + (2 * $radius),
+				'strand'    => $strand,
+				'position'  => $position, 
 				'value'     => 'score',
 			);
 			%test_pos2data = get_region_dataset_hash(
@@ -345,6 +506,8 @@ sub collect_correlations {
 				'chromo'    => $chromo,
 				'start'     => $ref_point - (2 * $radius),
 				'stop'      => $ref_point + (2 * $radius),
+				'strand'    => $strand,
+				'position'  => $position, 
 				'value'     => 'score',
 			);
 		}
@@ -359,7 +522,7 @@ sub collect_correlations {
 				'start'     => $start,
 				'stop'      => $stop,
 				'strand'    => $strand,
-				'position'  => 5, 
+				'position'  => $position, 
 				'value'     => 'score',
 			);
 			%test_pos2data = get_region_dataset_hash(
@@ -370,7 +533,7 @@ sub collect_correlations {
 				'start'     => $start,
 				'stop'      => $stop,
 				'strand'    => $strand,
-				'position'  => 5, 
+				'position'  => $position, 
 				'value'     => 'score',
 			);
 		}
@@ -390,7 +553,8 @@ sub collect_correlations {
 			)
 		) {
 			# not enough data points to work with
-			$mainData->{'data_table'}->[$row][$index{r}]       = '.';
+			$mainData->{'data_table'}->[$row][$index{r}] = '.';
+			$mainData->{'data_table'}->[$row][$index{p}] = '.' if ($find_pvalue);
 			if ($find_shift) {
 				$mainData->{'data_table'}->[$row][$index{shiftval}] = '.';
 				$mainData->{'data_table'}->[$row][$index{shiftr}]  = '.';
@@ -400,6 +564,13 @@ sub collect_correlations {
 		}
 			
 		
+		# Calculate Paired T-Test P value
+		# going to try this before normalizing
+		if ($find_pvalue) {
+			$mainData->{'data_table'}->[$row][$index{p}] = 
+				calculate_ttest(\%ref_pos2data, \%test_pos2data);
+		}
+		
 		
 		# Interpolate data points
 		# this will improve calculating Pearson values 
@@ -455,7 +626,8 @@ sub collect_correlations {
 		# verify the reference data
 		if ( min(@ref_data) == max(@ref_data) ) {
 			# no variance in the data! cannot calculate Pearson
-			$mainData->{'data_table'}->[$row][$index{r}]       = '.';
+			$mainData->{'data_table'}->[$row][$index{r}] = '.';
+			$mainData->{'data_table'}->[$row][$index{p}] = '.' if ($find_pvalue);
 			if ($find_shift) {
 				$mainData->{'data_table'}->[$row][$index{shiftval}] = '.';
 				$mainData->{'data_table'}->[$row][$index{shiftr}]  = '.';
@@ -522,28 +694,9 @@ sub collect_correlations {
 		$count++;
 	}
 	
-	
-	
-	# Summary analyses
-	printf " Correlated %s features\n", format_with_commas($count);
-	printf " Mean Pearson correlation was %.3f  +/- %.3f\n", 
-		mean(@correlations), stddevp(@correlations);
-	
-	if ($find_shift) {
-		printf " Mean absolute optimal shift was %.0f +/- %.0f bp\n", 
-			mean( map {abs $_} @optimal_shifts), 
-			stddevp( map {abs $_} @optimal_shifts);
-		printf " Mean optimal Pearson correlation was %.3f  +/- %.3f\n", 
-			mean(@optimal_correlations), stddevp(@optimal_correlations);
-	}
-	if ($not_enough_data) {
-		printf " %s features did not have enough data points\n", 
-			format_with_commas($not_enough_data);
-	}
-	if ($no_variance) {
-		printf " %s features had no variance in the reference data points\n", 
-			format_with_commas($no_variance);
-	}
+	# Return correlation results for summary
+	return (\@correlations, \@optimal_shifts, \@optimal_correlations,
+		$count, $not_enough_data, $no_variance);
 }
 
 
@@ -563,6 +716,7 @@ sub add_new_columns {
 	# extra information
 	$mainData->{$r_i}{'data_database'} = $data_database if ($data_database);
 	$mainData->{$r_i}{'normalization'} = $norm_method if ($norm_method);
+	$mainData->{$r_i}{'interpolate'}   = 1 if $interpolate;
 	if ($radius) {
 		$mainData->{$r_i}{'radius'} = $radius;
 		$mainData->{$r_i}{'reference_point'} = $position == 5 ? 
@@ -608,8 +762,24 @@ sub add_new_columns {
 		$mainData->{$shiftr_i}{'normalization'} = $norm_method if ($norm_method);
 	}
 	
+	# Paired Student T-Test P-value
+	my $p_i;
+	if ($find_pvalue) {
+		$p_i = $mainData->{'number_columns'};
+		$mainData->{$p_i} = {
+			'index'     => $p_i,
+			'name'      => 'ANOVA_Pval',
+			'reference' => $refDataSet,
+			'test'      => $testDataSet,
+			'transform' => '-Log10',
+		};
+		$mainData->{'data_table'}->[0][$p_i] = 'ANOVA_Pval';
+		$mainData->{'number_columns'} += 1;
+	}
+	
 	# record the indices
 	$index{r} = $r_i;
+	$index{p} = $p_i;
 	$index{shiftval} = $shift_i;
 	$index{shiftr} = $shiftr_i;
 }
@@ -786,7 +956,7 @@ sub calculate_optimum_shift {
 		
 		# calculate Pearson correlation
 		my $pearson = calculate_pearson($ref_data, \@test_data); 
-		if (defined $pearson) {
+		if (defined $pearson and $pearson > 0.6) {
 			$shift2pearson{$shift} = $pearson;
 		}
 	}
@@ -894,8 +1064,70 @@ sub calculate_pearson {
 }
 
 
+sub calculate_ttest {
+	
+	# generate the data arrays
+	my ($ref, $test) = @_;
+	my @ref_data;
+	my @test_data;
+	
+	# load the data arrays
+	my $min_value = min( min(keys %{$ref}), min(keys %{$test}) );
+	my $max_value = max( max(keys %{$ref}), max(keys %{$test}) );
+	for my $i ($min_value .. $max_value) {
+		push @ref_data, $ref->{$i} || 0;
+		push @test_data, $test->{$i} || 0;
+	}
+	
+	# calculate ANOVA
+	my $p_value;
+	my $aov = Statistics::ANOVA->new();
+	eval {
+		$aov->load( {
+			'test' => \@test_data,
+			'ref'  => \@ref_data,
+		} );
+		$aov->anova(
+			'independent'   => $INDEPENDENCE, 
+			'parametric'    => $PARAMETRIC,
+			'ordinal'       => $ORDINAL,
+		);
+		$p_value = $aov->{_stat}{p_value};
+	};
+	
+	$p_value = -1 * (log($p_value) / LOG10) if $p_value;
+	return $p_value || '.';
+}
 
 
+sub summarize_results {
+	
+	# the results from the correlation analysis
+	my ($correlations, $optimal_shifts, $optimal_correlations, 
+		$count, $not_enough_data, $no_variance) = @_;
+	
+	# Summary analyses
+	printf " Correlated %s features\n", format_with_commas($count);
+	printf " Mean Pearson correlation was %.3f  +/- %.3f\n", 
+		mean(@$correlations), stddevp(@$correlations);
+	
+	if ($find_shift) {
+		printf " Mean absolute optimal shift was %.0f +/- %.0f bp\n", 
+			mean( map {abs $_} @$optimal_shifts), 
+			stddevp( map {abs $_} @$optimal_shifts);
+		printf " Mean optimal Pearson correlation was %.3f  +/- %.3f\n", 
+			mean(@$optimal_correlations), stddevp(@$optimal_correlations);
+	}
+	if ($not_enough_data) {
+		printf " %s features did not have enough data points\n", 
+			format_with_commas($not_enough_data);
+	}
+	if ($no_variance) {
+		printf " %s features had no variance in the reference data points\n", 
+			format_with_commas($no_variance);
+	}
+}
+
 
 
 __END__
@@ -917,10 +1149,11 @@ correlate_position_data.pl [--options] <filename>
   --ddb <name | filename>
   --ref <type | filename>
   --test <type | filename>
+  --pval
   --shift
   --radius <integer>
-  --pos [5 | m | 3]
-  --norm [rank | sum ]
+  --pos [5|m|3]                 (m)
+  --norm [rank|sum ]
   --force_strand
   --(no)interpolate
   --gz
@@ -971,6 +1204,16 @@ correlate. These may be GFF type or name in a database or BigWigSet, or
 they may be a BigWig or even Bam file. Both options are required. If 
 not provided, they may be interactively chosen from the database.
 
+=item --pval
+
+Perform an ANOVA analysis between the test and reference data sets and 
+report a P-value. By default, this performs a dependent, parametric 
+ANOVA. This requires the C<Statistic::ANOVA> module to be installed. 
+Please refer to the module documentation for details. If your needs 
+require a change to the test, you may edit the parameters at the top 
+of this script. For convenience, the P-values are reported as -Log10(P) 
+transformed values. The default is false.
+
 =item --shift
 
 Optionally specify whether an optimal shift should be calculated that 
@@ -985,7 +1228,7 @@ when calculating an optimal shift (--shift option). The default is to
 take the length of the feature as the window for calculating the 
 correlation.  
 
-=item --pos [5 | m | 3]
+=item --pos [5|m|3]
 
 Indicate the relative position of the feature to be used as the 
 reference point around which the window (determined by the radius 
@@ -994,7 +1237,7 @@ accepted: "5" indicates the 5' prime end is used, "3" indicates the
 3' end is used, and "m" indicates the middle of the feature is used. 
 The default is to use the midpoint. 
 
-=item --norm [rank | sum]
+=item --norm [rank|sum]
 
 Optionally define a method of normalizing the scores between the 
 reference and test data sets prior to calculating the correlation. 
@@ -1035,13 +1278,18 @@ Display this POD documentation.
 
 =head1 DESCRIPTION
 
-This program will calculate a Pearson linear correlation coefficient (r value) 
-between the positioned scores (occupancy) of two datasets over a window 
-from an annotated feature or chromosomal segment. This will determine 
-whether the positions or distribution of scores across the window vary 
-between two different data sets: a test dataset and a reference dataset. 
-The original implementation of this program is to compare nucleosome 
-occupancy differences between two datasets and identify shifts in position. 
+This program will calculate statistics between the positioned scores of
+two different datasets over a window from an annotated feature or
+chromosomal segment. These statistics will help determine whether the
+positions or distribution of scores across the window vary or underwent
+a positional shift between a test and a reference dataset. For example,
+if the enrichment of nucleosome signal from a ChIP experiment shifts in
+genomic position, indicating a change in nucleosome position. 
+
+Two statistics may be calculated. First, it will calculate a a Pearson
+linear correlation coefficient (r value) between the datasets (default). 
+Additionally, an ANOVA analysis may be performed between the datasets and 
+generate a P-value.  
 
 By default, the correlation is determined between the data points 
 collected over the entire length of the feature. Alternatively, a 
@@ -1050,9 +1298,10 @@ that sets the window for collecting scores and calculating a correlation.
 
 To ensure a more reliable Pearson value, missing values or values of 
 zero are interpolated from neighboring values, when possible. Also, 
-values may be normalized using one of three methods. The values may be 
+values may be normalized using one of two methods. The values may be 
 converted to rank positions (compare to Kendall's tau), or scaled such 
-that the absolute sum or median values are equal.
+that the absolute sum values are equal (for example, when working with 
+sequence tag read counts).
 
 In addition to calculating a correlation coefficient, an optimal shift 
 may also be calculated. This essentially shifts the data, 1 bp at a time, 
