@@ -10,7 +10,7 @@ use Bio::ToolBox::file_helper qw(
 	write_tim_data_file
 	open_to_write_fh
 );
-my $VERSION = '1.15';
+my $VERSION = '1.18';
 
 print "\n This script will split a data file by features\n\n";
 
@@ -33,6 +33,7 @@ my (
 	$index,
 	$max,
 	$gz,
+	$prefix,
 	$help,
 	$print_version,
 );
@@ -42,6 +43,7 @@ GetOptions(
 	'in=s'        => \$infile, # specify the input data file
 	'index|col=i' => \$index, # index for the column to use for splitting
 	'max=i'       => \$max, # maximum number of lines per file
+	'prefix=s'    => \$prefix, # output file prefix
 	'gz!'         => \$gz, # compress output files
 	'help'        => \$help, # request help
 	'version'     => \$print_version, # print the version
@@ -82,7 +84,6 @@ unless (defined $gz) {
 }
 
 
-
 ### Load file
 my ($in_fh, $metadata_ref) = open_tim_data_file($infile);
 unless ($in_fh) {
@@ -314,9 +315,20 @@ sub request_new_file_name {
 	# calculate a new file name based on the current check value and part number
 	my $value = shift;
 	my $filename_value = $value;
-	$filename_value =~ s/[\:\|\\\/\+\*\?\# ]+/_/g; # replace unsafe characters
-	my $file = $metadata_ref->{'path'} . $metadata_ref->{'basename'} . 
+	$filename_value =~ s/[\:\|\\\/\+\*\?\#\(\)\[\]\{\} ]+/_/g; 
+		# replace unsafe characters
+	
+	my $file;
+	if ($prefix and $prefix eq 'none') {
+		$file = $metadata_ref->{'path'} . $filename_value;
+	}
+	elsif ($prefix) {
+		$file = $prefix . '#' . $filename_value;
+	}
+	else {
+		$file = $metadata_ref->{'path'} . $metadata_ref->{'basename'} . 
 		'#' . $filename_value;
+	}
 	
 	# add the file part number, if we're working with maximum line files
 	# padded for proper sorting
@@ -367,6 +379,7 @@ split_data_file.pl [--options] <filename>
   --in <filename>
   --index <column_index>
   --max <integer>
+  --prefix <text>
   --gz
   --version
   --help
@@ -398,6 +411,12 @@ file. Each group of specific value data is written to one or more files.
 Enter as an integer; underscores may be used as thousands separator, e.g. 
 100_000. 
 
+=item --prefix <text>
+
+Optionally provide a filename prefix for the output files. The default 
+prefix is the input filename base name. If no prefix is desired, using 
+just the values as filenames, then set the prefix to 'none'.
+
 =item --gz
 
 Indicate whether the output files should be compressed 
